// gshhs_help.cxx
// Show command line HELP, and command line DECODE

#include "gshhs_win.h"

#define  ISNUM(a) (( a >= '0' ) && ( a <= '9' ))

char * def_args[4];

static char filedate[] = __DATE__;
static char filetime[] = __TIME__;
static char szTmpBuf[1024];

char * Get_Version( void )
{
   static char vers_buf[128];
   char * ptmp = vers_buf;
   int i = sprintf(ptmp, "Version: %d.%d.%d of %s at %s (Base %s)",
      GSHHS_WIN_MAJ_VERSION, GSHHS_WIN_MIN_VERSION, GSHHS_WIN_PT_VERSION,
      filedate, filetime,
      GSHHS_PROG_VERSION );
   return ptmp;
}

void Get_File_Name_Only( PTSTR dest, PTSTR src )
{
   size_t len = strlen(src);
   size_t i;
   PTSTR pbgn = src;
   for( i = 0; i < len; i++ )
   {
      int c = src[i];
      if(( c == ':' )||( c == '\\')||( c == '/')) {
         pbgn = &src[i+1];
      }
   }
   strcpy(dest,pbgn);
   len = strlen(dest);
   for( i = 0; i < len; i++ )
   {
      int c = dest[i];
      if(( c == '.' )&&(strcmpi(&dest[i+1], "exe") == 0)) {
         dest[i] = 0;
         break;
      }
   }
}

void show_help( char * pn, int ex, PGSINFO pgsi )
{
   PBMPINFO pbmi = &pgsi->bmpinfo;
   char * ptmp = szTmpBuf;
   Get_File_Name_Only(ptmp, pn);
   sprtf ( "%s: %s\n", ptmp, Get_Version() );
   sprtf ( "GSHHS dataset ASCII and BITMAP export tool, and more ...\n" );
   sprtf ( "Usage: %s [Options] gshhs_[f|h|i|l|c].b [more_input_files]\n", ptmp);
   sprtf ( "Options, in any order, case insensitive ...\n" );
   sprtf ( " -? (or -h[elp])     - This brief HELP.\n" );

   sprtf ( " -addscale           - Add a scale legend to the bottom left. (Def=%s)\n",
      (pgsi->bmpinfo.addscale ? "On" : "Off") );

   sprtf ( " -auto[=nnn[K]]      - Auto scale to produce User BMP of nnnKB. (Def=%s at %ldKB).\n",
      (pgsi->bmpinfo.range.autoscale ? "On" : "Off"),
      pgsi->bmpinfo.range.bmpsize );
   sprtf ( " -bmp=outfile.bmp    - Output results of search to BITMAP file. (%s)\n",
      pgsi->bmpinfo.bmpname[0] ? pgsi->bmpinfo.bmpname : "Off" );
   sprtf ( " -bpp=nn             - Set Bits Per Pixel, 1,4,8,24. (Def=%d)\n",
      pgsi->bmpinfo.nBPP );
   sprtf ( " -block[+|-]         - Write additional 'block' BMP (Def=%s)\n",
      ( pgsi->bmpinfo.blockwrite ? "On" : "Off" ));
   sprtf ( " -border[+|-]        - Add border to bitmap. (Def=%s)\n",
      ( pgsi->bmpinfo.addborder ? "On" : "Off" ));

   sprtf ( " -center=lat,lon,rng - Center on this lat,lon, for this range (in Km). (Def=%3.6f,%3.6f,%3.1f)\n",
      pbmi->range.centlat,
      pbmi->range.centlon,
      pbmi->range.range );

   sprtf ( " -cull[+|-]          - Perform culling to potentially reduce BMP size. (Def=%s)\n",
      ( pgsi->bmpinfo.doculling ? "On" : "Off" ));

#ifdef  WIN32
   sprtf ( " -exec[=program]     - Execute program %s to display bitmap. (Def=%s)\n",
      pgsi->bmpinfo.exename,
      (pgsi->bmpinfo.exename ? "On" : "Off") );
#endif // #ifdef  WIN32

#ifndef NO_ADD_FG_DATA
   sprtf ( " -fg=apt.dat.gz      - Add airports and runways from Flightgear data. (%s)\n",
      ( pgsi->fgdata.aptdat[0] ? pgsi->fgdata.aptdat : "Off" ));
   sprtf ( " -fg:all             - Add taxiways, as well as runways. (Def=%s)\n",
      ( pgsi->fgdata.add_all ? "On" : "Off" ));
   sprtf ( " -fg:fill            - Fill boundary with color. (Def=%s)\n",
      ( pgsi->fgdata.add_fill ? "On" : "Off" ));
   sprtf ( " -fg:name            - Add Airport NAME text. (Def=%s)\n",
      ( pgsi->fgdata.add_name ? "On" : "Off" ));
   sprtf ( " -fg:id              - Add Airport ID (ICAO) text. (Def=%s)\n",
      ( pgsi->fgdata.add_id ? "On" : "Off" ));
   sprtf ( " -fgnav=nav.dat.gz   - Add Navaids from FlightGear data. (%s)\n",
      ( pgsi->fgdata.navdat[0] ? pgsi->fgdata.navdat : "Off" ));
   sprtf ( " -fgnav:circle       - Add VOR/NDB circles, at range. (Def=%s)\n",
      ( pgsi->fgdata.add_circle ? "On" : "Off" ));
   sprtf ( " -fgnav:ident        - Add VOR/NDB ident string. (Def=%s)\n",
      ( pgsi->fgdata.add_ident ? "On" : "Off" ));
#endif // #ifndef NO_ADD_FG_DATA
   sprtf ( " -grid[+|-]          - Add lat/lon grid to bitmap. (Def=%s)\n",
      ( pgsi->bmpinfo.addgrid ? "On" : "Off" ));
   sprtf ( " -info[+|-]          - Add file information to bitmap. (def=%s)\n",
      ( pgsi->bmpinfo.addinfo ? "On" : "Off" ));

   sprtf ( " -join[+|-]          - Join GSHHS data points of same number. (def=%s)\n",
      ( pgsi->bmpinfo.joinGSHHS ? "On" : "Off" ));

   sprtf ( " -l[:ahp]            - List all - a = all, h = headers, p = point data output (All=%s, Hdrs=%s, Pts=%s)\n",
      ( pgsi->listinfoa ? "On" : "Off"),
      ( pgsi->listinfoh ? "On" : "Off"),
      ( pgsi->listinfop ? "On" : "Off"));

   sprtf ( " -maxlat=[-]nn.nnnn  - Maximum latitude. (Def=%3.1f north)\n",
      pbmi->range.n );
   sprtf ( " -minlat=[-]nn.nnnn  - Minimum latitude. (Def=%3.1f south)\n",
      pbmi->range.s );
   sprtf ( " -maxlon=[-]nn.nnnn  - Maximum longitude. (Def=%3.1f east)\n",
      pbmi->range.e );
   sprtf ( " -minlon=[-]nn.nnnn  - Minimum longitude. (Def=%3.1f west)\n",
      pbmi->range.w );
   sprtf ( " -p[-|+]             - Use Mercator projection, (Def=%s)\n",
      (pgsi->bmpinfo.mercator ? "On" : "Off") );
   sprtf ( " -scale=nn           - Bitmap scaling. (Def=%d%s).\n",
      pbmi->range.scale,
      (pgsi->bmpinfo.range.autoscale ? ", but auto is ON" : "") );

   sprtf ( " -v[0-9]             - Verbosity. (Def=%d)\n",
      pgsi->verbosity );

   sprtf ( "Any non-switched entry is treated as a file name. Multiple names allowed.\n" );
   sprtf ( "NOTES: Warning: Very large bitmaps can be created, but there are very few"MEOR );
   sprtf ( "       programs capable of displaying them. It is best to set max/min of"MEOR );
   sprtf ( "       the lat/lon, especially if an enlarged scale is used, like 80 or 100"MEOR );
   sprtf ( "       or more. No attempt has been made to compress the bitmaps created."MEOR );
   sprtf ( "       This is beyond the purpose of this program."MEOR );
   sprtf ( "                                                       Happy GSHHS'ing ;=))\n" );

}

void give_help( char * pn, int ex, PGSINFO pgsi )
{
   show_help( pn, ex, pgsi );
	pgm_exit (ex);
}


void Unknown_Cmd( char * pn, char * bopt, PGSINFO pgsi )
{
   sprtf( "\nBrief help -\n" );
   show_help( pn, 3, pgsi );
   sprtf( "\nERROR: Unknown command! [%s]\n", bopt );
   pgm_exit( 3 );
}

// get -auto[=nn[K]] - auto scale, and set minimum
// User BMP size in KB
// or -addscale[+|-]
void Get_A_Opt(  char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf;
   size_t len;
   int   c;
   BOOL  bval = TRUE;

   bopt--;  // back up to beginning of command
   strcpy(ptmp, cp);
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "AUTO", 4) == 0 ) {
      pgsi->bmpinfo.range.autoscale = 1;
      if( len > 4 ) {
         ptmp = &ptmp[4];
         c = *ptmp;
         if( c == '=' ) {
            ptmp++;  // get to the KB size
            c = *ptmp;
            if( ISNUM(c) ) {
               c = atoi(ptmp);
               if(c > 10) {
                  pgsi->bmpinfo.range.bmpsize = c;
               } else {
                  sprtf( "ERROR: Minimum KB is 10KB, got %d!\n", c );
                  goto Error_A;
               }
            } else {
               sprtf( "ERROR: Only NUMBER can follow -auto=!\n" );
               goto Error_A;
            }
         } else {
            goto Error_A;
         }
      }
   } else if( _strnicmp(ptmp, "addscale", 8) == 0 ) {
      bval = TRUE;
      if( len > 8 ) {
         ptmp = &ptmp[8];
         if( *ptmp == '+' )
            bval = TRUE;
         else if( *ptmp == '-' )
            bval = FALSE;
         else {
            sprtf( "ERROR: Only + or - can follow -addscale! Got %s?\n", ptmp );
            goto Error_A;
         }
      }
      pgsi->bmpinfo.addscale = bval;
      if( VERB8(pgsi) ) {
         sprtf( "v8: AddScale set %s ...\n",
            ( pgsi->bmpinfo.addscale ? "On" : "Off" ) );
      }
   } else {
Error_A:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}

void Get_V_Opt(  char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf;
   char c;
   size_t len;
   bopt--;  // back up to beginning of command
   strcpy(ptmp, cp);
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "V", 1) == 0 ) {
      ptmp++;
      c = *ptmp;
      if( ISNUM(c) ) {
         pgsi->verbosity = atoi(ptmp);
      } else {
         sprtf( "ERROR: Only NUMBER can follow -v!\n" );
         goto Error_V;
      }
   } else {
Error_V:
      Unknown_Cmd( pn, bopt, pgsi );
   }
   if( VERB8(pgsi) ) {
      sprtf( "v8: Verbosity set to %d ...\n", pgsi->verbosity );
   }
}

#ifndef NO_ADD_FG_DATA
void Get_F_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   // get the FG file
   char * bopt = cp;
   char * ptmp = szTmpBuf;
   size_t len;
   int   c;
   BOOL  val = TRUE;
   PBOOL pbool;

   bopt--;  // back to beginning
   strcpy(ptmp, cp);
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "fg=", 3) == 0 ) {
      if( len > 3 ) {
         ptmp = &ptmp[3];
         c = *ptmp;
         if( File_Exists( ptmp ) ) {
            strcpy( pgsi->fgdata.aptdat, ptmp );
            InitLList( &pgsi->fgdata.apts );
            InitLList( &pgsi->fgdata.rwys );
         } else {
            sprtf("Error: Unable to locate file [%s] ...\n", ptmp );
            goto Error_F;
         }
      } else {
         sprtf("Error: File name should follow -fg=!\n" );
         goto Error_F;
      }
   } else if( _strnicmp(ptmp, "fg:", 3) == 0 ) {
      char * fgopts = "all|fill|name|id";
      if( len > 3 ) {
         ptmp = &ptmp[3];
         val = TRUE;
         pbool = &pgsi->fgdata.add_all;
         if( _strnicmp( ptmp, "all", 3 ) == 0 ) {
            pbool = &pgsi->fgdata.add_all;
            ptmp = &ptmp[3];
         } else if( _strnicmp( ptmp, "fill", 4 ) == 0 ) {
            pbool = &pgsi->fgdata.add_fill;
            ptmp = &ptmp[4];
         } else if( _strnicmp( ptmp, "name", 4 ) == 0 ) {
            pbool = &pgsi->fgdata.add_name;
            ptmp = &ptmp[4];
         } else if( _strnicmp( ptmp, "id", 2 ) == 0 ) {
            pbool = &pgsi->fgdata.add_id;
            ptmp = &ptmp[2];
         } else {
            goto FG_ERR;
         }
         if( *ptmp ) {
            if(*ptmp == '-')
               val = FALSE;
            else if(*ptmp == '+')
               val = TRUE;
            else {
               sprtf("Error: Only option + or - can follow '%s' for -fg:!\n", fgopts );
               goto Error_F;
            }
         }
         *pbool = val;
      } else {
FG_ERR:
         sprtf("Error: Options, like '%s' should follow -fg:!\n", fgopts );
         goto Error_F;
      }
   } else if( _strnicmp(ptmp, "fgnav=", 6) == 0 ) {
      if( len > 6 ) {
         ptmp = &ptmp[6];
         if( File_Exists( ptmp ) ) {
            strcpy( pgsi->fgdata.navdat, ptmp );
            InitLList( &pgsi->fgdata.navs );
         } else {
            sprtf("Error: Unable to locate file [%s] ...\n", ptmp );
            goto Error_F;
         }
      } else {
         sprtf("Error: Option, -fgnav= must be followed by a file name!\n" );
         goto Error_F;
      }
   } else if( _strnicmp(ptmp, "fgnav:", 6) == 0 ) {
      if( len > 6 ) {
         ptmp = &ptmp[6];
         pbool = &pgsi->fgdata.add_circle;
         val = TRUE;
         if( _strnicmp( ptmp, "circle", 6 ) == 0 ) {
            pbool = &pgsi->fgdata.add_circle;
            ptmp = &ptmp[6];
         } else if( _strnicmp( ptmp, "ident", 5 ) == 0 ) {
            pbool = &pgsi->fgdata.add_ident;
            ptmp = &ptmp[5];
         } else {
            goto Nav_Err;
         }
         if( *ptmp ) {
            if(*ptmp == '-')
               val = FALSE;
            else if(*ptmp == '+')
               val = TRUE;
            else
               goto Nav_Err;
         }
         *pbool = val;
      } else {
Nav_Err:
         sprtf("Error: Option, -fgnav: must be followed by 'circle[+|-]' or 'ident[+|-]'!\n" );
         goto Error_F;
      }
   } else {
Error_F:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}
#endif // #ifndef NO_ADD_FG_DATA

void Get_G_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf;
   char c;
   size_t len;
   PBOOL pb = &pgsi->bmpinfo.addgrid;
   BOOL  val = TRUE;
   bopt--;  // back up to beginning of command
   strcpy(ptmp, cp);
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "grid", 4) == 0 ) {
      if( len > 4 ) {
         ptmp = &ptmp[4];
         c = *ptmp;
         if( c == '-' )
            val = FALSE;
         else if( c == '+' )
            val = TRUE;
         else {
            sprtf( "ERROR: Only + (on) or - (off) can follow -grid ...\n" );
            goto Error_G;
         }
      }
      *pb = val;
      if( VERB8(pgsi) ) {
         sprtf( "v8: Grid set to %s ...\n", (val ? "On" : "Off") );
      }
   } else {
Error_G:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}

void Get_I_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf;
   char c;
   size_t len;
   PBOOL pb = &pgsi->bmpinfo.addinfo;
   BOOL  val = TRUE;
   bopt--;  // back up to beginning of command
   strcpy(ptmp, cp);
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "info", 4) == 0 ) {
      if( len > 4 ) {
         ptmp = &ptmp[4];
         c = *ptmp;
         if( c == '-' )
            val = FALSE;
         else if( c == '+' )
            val = TRUE;
         else {
            sprtf( "ERROR: Only + (on) or - (off) can follow -info ...\n" );
            goto Error_G;
         }
      }
      *pb = val;
      if( VERB8(pgsi) ) {
         sprtf( "v8: Info set to %s ...\n", (val ? "On" : "Off") );
      }
   } else {
Error_G:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}

void Get_J_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf;
   char c;
   size_t len;
   PBOOL pb = &pgsi->bmpinfo.joinGSHHS;
   BOOL  val = TRUE;
   bopt--;  // back up to beginning of command
   strcpy(ptmp, cp);
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "join", 4) == 0 ) {
      if( len > 4 ) {
         ptmp = &ptmp[4];
         c = *ptmp;
         if( c == '-' )
            val = FALSE;
         else if( c == '+' )
            val = TRUE;
         else {
            sprtf( "ERROR: Only + (on) or - (off) can follow -join ...\n" );
            goto Error_J;
         }
      }
      *pb = val;
      if( VERB8(pgsi) ) {
         sprtf( "v8: Join GSHHS points set to %s ...\n", (val ? "On" : "Off") );
      }
   } else {
Error_J:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}

void Get_B_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf; 
   char * p;
   char c;
   int   val;
   BOOL  bval = TRUE;
   size_t len;

   bopt--;  // back up to beginning of command
   strcpy(ptmp,cp);  // copy it, for modification
   len = strlen(ptmp);
   p = strchr( ptmp, '=' );
   if(p) {
      *p = 0;
      p++;
      if((strcmpi(ptmp, "bmp") == 0) && *p) {
         strcpy(pgsi->bmpinfo.bmpname, p);
         if( VERB8(pgsi) ) {
            sprtf( "v8: BMP output to %s ...\n", pgsi->bmpinfo.bmpname );
         }
      } else if(( strcmpi(ptmp, "bpp") == 0) && *p) {
         val = atoi(p);
         if((val == 1)||(val == 4)||(val == 24)||(val == 8)) {
            pgsi->bmpinfo.nBPP = val;
            if( VERB8(pgsi) ) {
               sprtf( "v8: BPP set to %d ...\n", pgsi->bmpinfo.nBPP );
            }
         } else {
            sprtf( "ERROR: Presently only 1, 4, 8, or 24 can follow -bpp!\n" );
            goto Error_B;
         }
      } else {
         goto Error_B;
      }

   } else if( _strnicmp(ptmp, "border", 6) == 0  ) {
      PBOOL pb = &pgsi->bmpinfo.addborder;
      if( len > 6 ) {
         ptmp = &ptmp[6];
         c = *ptmp;
         if( c == '-' )
            bval = FALSE;
         else if( c == '+' )
            bval = TRUE;
         else {
            sprtf( "ERROR: Only + (on) or - (off) can follow -border ...\n" );
            goto Error_B;
         }
      }
      *pb = bval;
      if( VERB8(pgsi) ) {
         sprtf( "v8: Add border set to %s ...\n", (bval ? "On" : "Off") );
      }
   } else if( _strnicmp(ptmp, "block", 5) == 0  ) {
      PBOOL pb = &pgsi->bmpinfo.blockwrite;
      bval = TRUE;
      if( len > 5 ) {
         ptmp = &ptmp[5];
         c = *ptmp;
         if( c == '-' )
            bval = FALSE;
         else if( c == '+' )
            bval = TRUE;
         else {
            sprtf( "ERROR: Only + (on) or - (off) can follow -block ...\n" );
            goto Error_B;
         }
      }
      *pb = bval;
      if( VERB8(pgsi) ) {
         sprtf( "v8: Add block set to %s ...\n", (bval ? "On" : "Off") );
      }
   } else {
Error_B:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}

BOOL  Is_Double_Number( char * ps )
{
   BOOL  bret = FALSE;
   size_t len = strlen(ps);
   size_t i;
   int   c;
   if(len) {
      for( i = 0; i < len; i++ )
      {
         c = ps[i];
         if( !( ISNUM(c) ||
            (( c == '+' )&&( i == 0 )) ||
            (( c == '-' )&&( i == 0 )) ||
            ( c == '.' ) ) )
            return FALSE;
      }
      bret = TRUE;
   }
   return bret;
}


void Get_C_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf; 
   char * p;
   char * p2;
   char c;
   //int   val;
   BOOL  bval = TRUE;
   size_t len;

   bopt--;  // back up to beginning of command
   strcpy(ptmp,cp);  // copy it, for modification
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "cull", 4) == 0  ) {
      PBOOL pb = &pgsi->bmpinfo.doculling;
      bval = TRUE;
      if( len > 4 ) {
         ptmp = &ptmp[4];
         c = *ptmp;
         if( c == '-' )
            bval = FALSE;
         else if( c == '+' )
            bval = TRUE;
         else {
            sprtf( "ERROR: Only + (on) or - (off) can follow -cull ...\n" );
            goto Error_C;
         }
      }
      *pb = bval;
      if( VERB8(pgsi) ) {
         sprtf( "v8: Culling set to %s ...\n", (bval ? "On" : "Off") );
      }
   } else if( _strnicmp(ptmp, "center=", 7) == 0  ) {
      ptmp = &ptmp[7];  // expect lat,lon, comma separated
      p = strchr(ptmp,',');
      if(p) {
         p2 = strchr( &p[1], ',' );
         if( !p2 ) goto Cent_Err;
         *p = 0;
         *p2 = 0;
         p++;
         p2++;
         if( Is_Double_Number(ptmp) && Is_Double_Number(p) && Is_Double_Number(p2)) {
            int res;
            _CRT_DOUBLE crtd;
            res = _atodbl(&crtd, ptmp);
            if(res) {
               sprtf( "ERROR: Failed to get double value! %s\n",
                  ((res == _UNDERFLOW) ? "(Underflow)" :
                  (res == _OVERFLOW) ? "(Overflow)" : "(Unknown)"));
               goto Error_C;
            }
            // pbmi->range.range = 100 * 1000;     // 100 Km range
            pgsi->bmpinfo.range.centlat = crtd.x;
            res = _atodbl(&crtd, p);
            if(res) {
               sprtf( "ERROR: Failed to get double value! %s\n",
                  ((res == _UNDERFLOW) ? "(Underflow)" :
                  (res == _OVERFLOW) ? "(Overflow)" : "(Unknown)"));
               goto Error_C;
            }
            // pbmi->range.range = 100 * 1000;     // 100 Km range
            pgsi->bmpinfo.range.centlon = crtd.x;
            res = _atodbl(&crtd, p2);
            if(res) {
               sprtf( "ERROR: Failed to get double value! %s\n",
                  ((res == _UNDERFLOW) ? "(Underflow)" :
                  (res == _OVERFLOW) ? "(Overflow)" : "(Unknown)"));
               goto Error_C;
            }
            // pbmi->range.range = 100 * 1000;     // 100 Km range
            pgsi->bmpinfo.range.range = crtd.x;
            if( VERB8(pgsi) ) {
               sprtf( "v8: Center set to lat,lon %f,%f, range %f ...\n",
                  pgsi->bmpinfo.range.centlat,
                  pgsi->bmpinfo.range.centlon,
                  pgsi->bmpinfo.range.range );
            }
         } else {
Cent_Err:
            sprtf( "ERROR: lat,lon,range (doubles) must follow -center=, got %s,%s? ...\n",
               ptmp, p );
            goto Error_C;
         }
      } else {
         sprtf( "ERROR: lat,lon (doubles) must follow -center=, got %s? ...\n", ptmp );
         goto Error_C;
      }
   } else {
Error_C:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}

void Get_E_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf; 
   char * p;
   int   c;
   //int   val;
   BOOL  bval = TRUE;
   size_t len;
   PBOOL    pbool = &pgsi->bmpinfo.execute;

   bopt--;  // back up to beginning of command
   strcpy(ptmp,cp);  // copy it, for modification
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "exec", 4) == 0  ) {
      ptmp = &ptmp[4];
      c = *ptmp;
      if( c == '+' )
         bval = TRUE;
      else if ( c == '-' )
         bval = FALSE;
      else if ( c == '=' ) {
         p = &ptmp[1];  // get to FILE NAME
         if(*p) {
            strcpy( pgsi->bmpinfo.exename, p );
         } else {
            goto Error_E;
         }
      } else {
         sprtf( "ERROR: +, -, or =filename MUST follow -exec command!\n" );
         goto Error_E;
      }
      *pbool = bval;
   } else {
Error_E:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}



typedef BOOL (*INRANGE)( double );

BOOL  Lat_In_Range( double lat )
{
   if(( lat >= -90.0 )&&
      ( lat <=  90.0 ))
      return TRUE;

   return FALSE;
}

BOOL  Lon_In_Range( double lat )
{
   if(( lat >= -180.0 )&&
      ( lat <=  180.0 ))
      return TRUE;

   return FALSE;
}

//   sprtf ( " -l[:ahp]            - List all - a = all, h = headers, p = point data output (All=%s, Hdrs=%s, Pts=%s)\n",
//      ( pgsi->listinfoa ? "On" : "Off"),
//      ( pgsi->listinfoh ? "On" : "Off"),
//      ( pgsi->listinfop ? "On" : "Off"));
void Get_L_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf; 
   size_t len;
   char * p;
   int   c;
   BOOL  val = TRUE;
   PBOOL pbool = &pgsi->listinfoa;
   bopt--;  // back up to beginning of command
   strcpy(ptmp,cp);  // copy it, for modification
   len = strlen(ptmp);
   if( len > 1 ) {
      p = &ptmp[1];
      c = toupper(*p);
      if(c == ':') {  // got a COLON
         p++;  // goto NEXT
         c = toupper(*p);
         if(c) {
            while(c) {
               switch(c)
               {
               case 'A':
                  pbool = &pgsi->listinfoa;
                  break;
               case 'H':
                  pbool = &pgsi->listinfoh;
                  break;
               case 'P':
                  pbool = &pgsi->listinfop;
                  break;
               default:
                  goto Error_L;
                  break;
               }
               val = TRUE; // assume TRUE
               p++;        // move to NEXT
               c = toupper(*p);  // get char
               if( c == '-' ) {
                  val = FALSE;
                  p++;
                } else if( c == '+' ) {
                   val = TRUE;
                   p++;
               }
               *pbool = val;
               c = toupper(*p);  // get NEXT char
            }
         } else {
            sprtf( "ERROR: A, H or P must follow -l:, got blank?\n" );
            goto Error_L;
         }
      } else if((len == 2)&&(( c == '+' )||( c == '-'))) {
         // have -l+ or -l-
         if( c == '+' ) {
            val = TRUE;
            *pbool = val;
         } else {
            val = FALSE;
            *pbool = val;
            pgsi->listinfoh = FALSE;
            pgsi->listinfop = FALSE;
         }
      } else {
Error_L:
         Unknown_Cmd( pn, bopt, pgsi );
      }
   } else {
      // take a single -l, as meaning ALL
      *pbool = val;  // pgsi->listinfoa = 1;
   }

}

void Get_M_Opt( char * pn, char * cp, PGSINFO pgsi )
{
   char * bopt = cp;
   char * ptmp = szTmpBuf; 
   char * p;
   double * pd;
   int res;
   _CRT_DOUBLE crtd;
   INRANGE  ir = NULL;
   PBMPINFO pbmi = &pgsi->bmpinfo;

   bopt--;  // back up to beginning of command
   strcpy(ptmp,cp);  // copy it, for modification
   p = strchr( ptmp, '=' );
   if(p) {
      *p = 0;
      p++;
      if( strcmpi(ptmp, "maxlat") == 0 ) {
         pd = &pbmi->range.n;
         ir = Lat_In_Range;
      } else if( strcmpi(ptmp, "minlat") == 0 ) {
         pd = &pbmi->range.s;
         ir = Lat_In_Range;
      } else if( strcmpi(ptmp, "maxlon") == 0 ) {
         pd = &pbmi->range.e;
         ir = Lon_In_Range;
      } else if( strcmpi(ptmp, "minlon") == 0 ) {
         pd = &pbmi->range.w;
         ir = Lon_In_Range;
      } else {
         goto Error_M;
      }
      res = _atodbl(&crtd, p);
      if(res) {
         sprtf( "ERROR: Failed to get double value! %s\n",
            ((res == _UNDERFLOW) ? "(Underflow)" :
            (res == _OVERFLOW) ? "(Overflow)" : "(Unknown)"));
         goto Error_M;
      } else {
         if(ir) {
            if (ir(crtd.x)) {
               *pd = crtd.x;
               if( VERB8(pgsi) ) {
                  sprtf( "v8: %s set to %f ...\n", ptmp, *pd );
               }
            } else {
               sprtf( "ERROR: Out of WORLD range, value %3.3f!\n", crtd.x );
               goto Error_M;
            }
         }
      }
   } else {
Error_M:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}

BOOL  Is_All_Numbers( char * ps )
{
   BOOL  bret = FALSE;
   size_t len = strlen(ps);
   size_t i;
   if(len) {
      for( i = 0; i < len; i++ )
      {
         if( !ISNUM(ps[i]) )
            return FALSE;
      }
      bret = TRUE;
   }
   return bret;
}

void Get_P_Opt( char * pn, char * bopt, char * cp, PGSINFO pgsi )
{
   char * ptmp = szTmpBuf;
   char c;
   size_t len;
   PBOOL pb = &pgsi->bmpinfo.mercator;
   BOOL  val = TRUE;

   strcpy(ptmp, cp);
   len = strlen(ptmp);
   if( _strnicmp(ptmp, "p", 1) == 0 ) {
      if( len > 1 ) {
         ptmp = &ptmp[1];
         c = *ptmp;
         if( c == '-' )
            val = FALSE;
         else if( c == '+' )
            val = TRUE;
         else {
            sprtf( "ERROR: Only + (on) or - (off) can follow -p (projections) ...\n" );
            goto Error_P;
         }
      }
      *pb = val;
      if( VERB8(pgsi) ) {
         sprtf( "v8: Projection (Mercator) set to %s ...\n", (val ? "On" : "Off") );
      }
   } else {
Error_P:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}


void Get_S_Opt( char * pn, char * bopt, char * cp, PGSINFO pgsi )
{
   PBMPINFO pbmi = &pgsi->bmpinfo;
   char * ptmp = szTmpBuf; 
   char * p;
   PINT pi = (PINT)&pbmi->range.scale;
   strcpy(ptmp,cp);  // copy it, for modification
   p = strchr( ptmp, '=' );
   if(p) {
      *p = 0;
      p++;
      if( strcmpi(ptmp, "scale") == 0 ) {
         if( *p && Is_All_Numbers(p) )
         {
            *pi = atoi(p);
            if( *pi > 0 ) {
               if( VERB8(pgsi) ) {
                  sprtf( "v8: Scale set to %d ...\n", *pi );
               }
            } else {
               sprtf( "ERROR: Scale must be greater than ZERO! (%d)...\n",
                  *pi );
               goto Error_S;
            }
         } else {
            sprtf( "ERROR: Only integral number(s) can follow -scale= ...\n" );
            goto Error_S;
         }
      } else {
         sprtf( "ERROR: Only -scale=nn allowed ...\n" );
         goto Error_S;
      }
   } else {
Error_S:
      Unknown_Cmd( pn, bopt, pgsi );
   }
}


typedef struct tagPISTR {
   HANDLE   hFile;
   INT      iCnt;
   PTSTR   pBuf;
   PTSTR * pArgs;
}PISTR, * PPISTR;
#define  MXARGS   128    // should be ENOUGH

BOOL  Getpis( char * pn, PPISTR ppis, LPTSTR lpf )
{
   BOOL     bRet = FALSE;
   PTSTR   lptmp;

   ppis->iCnt  = 0;
   ppis->pBuf  = 0;
   ppis->pArgs = 0;
   // read this input file
   ppis->hFile = CreateFile( lpf,
            GENERIC_READ,
            FILE_SHARE_READ,
            0,
            OPEN_EXISTING,
            FILE_ATTRIBUTE_NORMAL,
            0 );
   if( VFH( ppis->hFile ) ) {
      DWORD dws, dwh;
      dws = GetFileSize( ppis->hFile, &dwh );
      if( dws && !dwh ) {
         PTSTR   lpb;
         dwh = dws + (MXARGS * sizeof(LPVOID));
         lpb = (PTSTR)MALLOC(dwh);
         CHKMEM(lpb,dwh);
         if( lpb ) {
            ppis->pBuf = lpb; // this is for freeing the pointer
            if( ( ReadFile( ppis->hFile, lpb, dws, &dwh, 0) ) &&
                ( dws == dwh ) ) {
               LPTSTR * lpv;
               int      icnt;
               TCHAR    c, d;
               DWORD    dwi;

               lpv = (LPTSTR *) (LPTSTR) ( lpb + dws + 2 );
               icnt = 0;
               lpv[icnt++] = pn; // points back to ORIGINAL NAME
               // ==============================================
               for( dwh = 0; dwh < dws; dwh++ ) {
                  c = lpb[dwh];
                  if( c > ' ' ) {
                     if( c == ';' ) {
                        dwh++;
                        for( ; dwh < dws; dwh++ ) {
                           if( lpb[dwh] < ' ' )
                              break;
                        }
                     } else {
                        // begin of an ARGUMENT
                        lptmp = &lpb[dwh];
                        lpv[icnt] = lptmp; // &lpb[dwh];
                        d = *lptmp;    // get first char
                        c = d;
                        dwh++;
                        for( ; dwh < dws; dwh++ ) {
                           c = lpb[dwh];
                           if( c < ' ' )
                              break;
                           else if( ( d != '"' ) && // not "protect" by DOUBLE QUOTES
                              ( c == ';' ) ) // this is a comment tailing a command
                              break;
                        }
                        lpb[dwh] = 0;  // zero terminate the string
                        dwi = dwh - 1; // remove any trailing space
                        while(dwi) {
                           if( lpb[dwi] > ' ' )
                              break;
                           lpb[dwi--] = 0;
                        }
                        // FIX20010329 - Ensure if the argument is "encased"
                        // in double quotes, remove them NOW (like command)
                        dwi = (DWORD)strlen(lptmp);    // get length of the command
                        if( ( dwi > 2      ) &&
                           ( d == '"' ) &&
                           ( lptmp[dwi-1] == '"' ) ) {
                           dwi--;
                           lptmp[dwi] = 0;   // kill trailing DOUBLE QUOTE
                           dwi--;
                           lptmp++;          // bump past leading DOUBLE QUOTE
                           lpv[icnt] = lptmp; // set this new pointer
                        }
                        if( dwi ) {
                           icnt++;  // bump the argument count
                        }
                        if( c >= ' ' ) {
                           // ensure we go to the end of the line
                           dwh++;   // get past the ZERO - FIX20020324
                           for( ; dwh < dws; dwh++ ) {
                              c = lpb[dwh];
                              if( c < ' ' )
                                 break;
                           }
                        }
                     }
                  }
               }  // for the length of the read in buffer
               // =======================================
               if( icnt > 1 ) {
                  ppis->iCnt = icnt;
                  ppis->pArgs = lpv;
               }
               // note we quietly forget BLANK input files!!!
               bRet = TRUE;
            }
         }
      }
   }
   return bRet;
}

void Process_Args(int argc, char **argv, PGSINFO pgsi);

void Process_Input( char * pn, char * porg, char * cp, PGSINFO pgsi )
{
   PISTR is;
   PPISTR   ppis = &is;
   BOOL  failed = FALSE;
   ZeroMemory( ppis, sizeof(PISTR) );
   if( Getpis( pn, ppis, cp ) ) {
      if( ( ppis->iCnt > 1 ) && ( ppis->pArgs ) )
         Process_Args( ppis->iCnt, ppis->pArgs, pgsi );
      // note we quietly forget BLANK input files!!!
   } else {
      failed = TRUE;
   }
   if( ppis->pBuf )
      MFREE(ppis->pBuf);

   if( VFH( ppis->hFile ) )
      CloseHandle( ppis->hFile );

   if( failed ) {
      sprtf("ERROR: Failed to OPEN, and load file %s ...\n", cp );
      Unknown_Cmd( pn, porg, pgsi );
   }
}

void Process_Verb(int argc, char **argv, PGSINFO pgsi)
{
   int i;
   char * cp;
   char c;
   char * pn = argv[0];
   for( i = 1; i < argc; i++ ) {
      cp = argv[i];
      c = toupper(*cp);
      if(( c == '-' )||( c == '/')) {
         cp++;
         c = toupper(*cp);
         if( c == 'V' )
            Get_V_Opt( pn, cp, pgsi);
      }
   }
}

void Process_Args(int argc, char **argv, PGSINFO pgsi)
{
   int i;
   char * cp;
   char c;
   char * pn = argv[0];
   Process_Verb(argc, argv, pgsi); // Always pre-process VERBOSITY
   if( VERB2(pgsi) )
      sprtf( "Processing arguments ...\n" );

   for( i = 1; i < argc; i++ ) {
      cp = argv[i];
      c = toupper(*cp);
      if( VERB8(pgsi) ) {
         sprtf( "v8: Processing argument %s ...\n", cp );
      }
      if(( c == '-' )||( c == '/')) {
         cp++;
         c = toupper(*cp);
         switch(c)
         {
         case 'H':
         case '?':
            give_help( pn, 0, pgsi );
            break;
         case 'A':
            Get_A_Opt( pn, cp, pgsi );
            break;
         case 'B':
            Get_B_Opt( pn, cp, pgsi );
            break;
         case 'C':
            Get_C_Opt( pn, cp, pgsi );
            break;
         case 'E':
            Get_E_Opt( pn, cp, pgsi );
            break;
#ifndef NO_ADD_FG_DATA
         case 'F':
            Get_F_Opt( pn, cp, pgsi );
            break;
#endif // #ifndef NO_ADD_FG_DATA
         case 'G':
            Get_G_Opt( pn, cp, pgsi );
            break;
         case 'I':
            Get_I_Opt( pn, cp, pgsi );
            break;
         case 'J':
            Get_J_Opt( pn, cp, pgsi );
            break;
         case 'L':
            Get_L_Opt( pn, cp, pgsi );
            break;
         case 'M':
            Get_M_Opt( pn, cp, pgsi);
            break;
         case 'P':
            Get_P_Opt( pn, argv[i], cp, pgsi);
            break;
         case 'S':
            Get_S_Opt( pn, argv[i], cp, pgsi);
            break;
         case 'V':
            //Get_V_Opt( pn, cp, pgsi); // already done '-V[n]'
            break;
         default:
            sprtf( "ERROR: Option %c not cased!\n", c );
            Unknown_Cmd( pn, argv[i], pgsi );
            break;
         }
      } else if( c == '@' ) {
         // input file
         cp++;
         Process_Input( pn, argv[i], cp, pgsi );
      } else {
         // bare argument - assume a FILE NAME
         Add_2_File_List(cp); // add to list to process
      }
   }
}

// eof - gshhs_help.cxx
